Package: this.path
Version: 2.8.0
Date: 2026-04-09
License: MIT + file LICENSE
Title: Get Executing Script's Path
Description: Determine the path of the executing script. Compatible
        with several popular GUIs: 'Rgui', 'RStudio', 'Positron',
        'VSCode', 'Jupyter', 'Emacs', and 'Rscript' (shell). Compatible
        with several functions and packages: 'source()',
        'sys.source()', 'debugSource()' in 'RStudio',
        'compiler::loadcmp()', 'utils::Sweave()', 'box::use()',
        'knitr::knit()', 'plumber::plumb()', 'shiny::runApp()',
        'package:targets', and 'testthat::source_file()'.
Authors@R: person(given = "Iris",
                  family = "Simmons",
                  role = c("aut", "cre"),
                  email = "ikwsimmo@gmail.com")
Author: Iris Simmons [aut, cre]
Maintainer: Iris Simmons <ikwsimmo@gmail.com>
Depends: R (>= 2.15)
Suggests: utils, jsonlite, microbenchmark, rprojroot
Enhances: compiler, box, knitr, plumber, shiny, targets, testthat
URL: https://github.com/ArcadeAntics/this.path
BugReports: https://github.com/ArcadeAntics/this.path/issues
ByteCompile: TRUE
Biarch: TRUE
BuildManual: TRUE
Type: Package
NeedsCompilation: yes
Packaged: 2026-04-09 03:13:52 UTC; iris
Repository: CRAN
Date/Publication: 2026-04-10 08:50:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-26 11:03:41 UTC; unix
Archs: this.path.so.dSYM
