this.path : Get Executing Script's Path
Copyright (C) 2022-2024   Iris Simmons


In Rgui for Windows, this.path() works by looking at the windows
handles belonging to that R process.

For an open script, the windows handle will end with " - R Editor" OR
ANY SUPPORTED TRANSLATION INTO ANOTHER LANGUAGE. For a new script, the
windows handle will be "Untitled - R Editor", again possibly translated
into another language.

What we do in "main.R" is:

  * launch a new Rgui process in each supported language and each
    supported version of R

  * open an existing script and grab the translation of " - R Editor"

  * open a new script and grab the translation of "Untitled - R Editor"

We will then save that information into files
"r-editor_msvcrt_2_15.dat", ..., "r-editor_ucrt_4_2.dat", ...,
"untitled_msvcrt_2_15.dat", ..., and "untitled_ucrt_4_2.dat", ...

Some of the files are redundant because the translations did not change
from previous R versions, and in such cases we delete those files.
