##
## this.path : Get Executing Script's Path
## Copyright (C) 2024-2025   Iris Simmons
##


## Defunct in <pkgname> <version> (<date>)


# <old> <- function (<formals>)
# stop(.defunctError("<new>", "this.path", old = "<old>"))
